/***********************************************************************
	created:	7/3/2005
	author:		jacmoe
*************************************************************************/
/***************************************************************************
 *   Copyright (C) 2004 - 2010 Paul D Turner & The CEGUI Development Team
 *
 *   Permission is hereby granted, free of charge, to any person obtaining
 *   a copy of this software and associated documentation files (the
 *   "Software"), to deal in the Software without restriction, including
 *   without limitation the rights to use, copy, modify, merge, publish,
 *   distribute, sublicense, and/or sell copies of the Software, and to
 *   permit persons to whom the Software is furnished to do so, subject to
 *   the following conditions:
 *
 *   The above copyright notice and this permission notice shall be
 *   included in all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *   OTHER DEALINGS IN THE SOFTWARE.
 ***************************************************************************/
//////////////////////////////////////////////////////////////////////////
/*************************************************************************

This file contains CEGUI version defines
*************************************************************************/
//////////////////////////////////////////////////////////////////////////
#ifndef _CEGUIVersion_h_
#define _CEGUIVersion_h_

//////////////////////////////////////////////////////////////////////////
// Define CEGUI version
//////////////////////////////////////////////////////////////////////////
#define CEGUI_VERSION_MAJOR 0
#define CEGUI_VERSION_MINOR 8
#define CEGUI_VERSION_PATCH 7

// this is used to check consistency between runtime binary and headers
// used for compiling.  You should not generally use this in client code
// or rely on its value meaning anything in particular.
#define CEGUI_VERSION_ABI 208

#endif	// end of guard _CEGUIVersion_h_

